#!/bin/sh

# ******************************************************************************
#
#  (C) 2018 AhnLab, Inc. All rights reserved.
#  Any part of this source code can not be copied with any method without
#  prior written permission from the author or authorized person.
#
# ******************************************************************************

# COLORS
RED='\033[1;31m'
BLUE='\033[1;34m'
GR='\033[1;32m'
NC='\033[0m'

APP_NAME=$1
#INSTALL_PATH="`cat /etc/v3dl.install.conf | grep InstallPath | cut -c 13-`"
INSTALL_PATH=$2
CONF_FILE=$3

# includes util.sh to log
export LOG_FILE_NAME
LOG_FILE_NAME=`basename ${0}`
. ${INSTALL_PATH}/bin/util.sh


if [ "_${APP_NAME}" = "_" ]; then
	echo_log "missing arguments"
	exit -1
fi

print_log "START WATCH [${GR}${APP_NAME}${NC}]"
`echo $$ > /var/run/$APP_NAME.dog.pid`


# see if a process is running or not
check_process() {
	if [ "$2" != "" ]; then
		PREFIX="[${2}]"                
	else
		PREFIX=
	fi
	
	if [ "${1}" = "lighttpd" ]; then
		PID_FILE=$LIGHTTPD_PID
		WATCHDOG=
	else
		PID_FILE="/var/run/${1}.pid"
		WATCHDOG="/var/run/${1}.dog.pid"
	fi

	PID_FIND=
	if [ -f $PID_FILE ]; then
		PID=`cat ${PID_FILE}`
		if [ "${PID}" != "" ]; then
			if [ "`uname`" = "HP-UX" ]; then
				PID_FIND=`ps -p ${PID} | grep ${PID}`
			else
				PID_FIND=`ps -e | grep -w ${PID}`
			fi
		else
			PID_FIND=
		fi

		if [ "_" != "_${PID_FIND}" ]; then
			print_log "${PREFIX}   |--[check_process(PF)] [${1}(${PID})] ${GR}RUNNING${NC}"
		else
			error_log "${PREFIX}   |--[check_process(PF)] [${1}(${PID})] ${RED}NOT RUNNING${NC}"
		fi
	else
		PID=`ps -ef|grep /bin/${1}|grep -v watchdog.sh|grep -v grep|grep -v tail|grep -v cat|awk '{{ print $2 }}' `
		if [ "" != "${PID}" ]; then
			print_log "${PREFIX}   |--[check_process(ps)] [${1}(${PID})] ${GR}RUNNING${NC}"
		else
			error_log "${PREFIX}   |--[check_process(ps)] [${1}(${PID})] ${RED}NOT RUNNING${NC}"
		fi
	fi


}

# delete file  and check_file
delete_file() {
	if [ "$2" != "" ]; then
		PREFIX="[${2}]"                
	else
		PREFIX=
	fi

	if [ "$1" != "" ]; then
		FILE="${1}"
	else
		error_log "${PREFIX}[delete_file] file for deleting not specified : ${1}"
		return 1
	fi

	if [ -f $FILE ]; then
		print_log "${PREFIX}[delete_file] trying to delete file : '${FILE}'"
		rm -rf "${FILE}"
	else
		error_log "${PREFIX}[delete_file] file not found : '${FILE}'"
	fi
}

# see if a file is exist or not
check_file() {
	if [ "$2" != "" ]; then
		PREFIX="[${2}]"                
	else
		PREFIX=
	fi
	
	if [ "$1" != "" ]; then
		FILE="${1}"
	else
		error_log "$PREFIX[check_file] file for check not specified : ${1}"
		return 1
	fi

	if [ -f $FILE ]; then
		print_log "$PREFIX[check_file] file found : '${FILE}'"
	else
		print_log "$PREFIX[check_file] file not found : '${FILE}'"
	fi
}

# call delete_file and then check_file
delete_file_check_file() {
	delete_file "${1}" "${2}"
 	check_file "${1}" "${2}"
}

# terminating process with SIGTERM first, if not SIGKILL
try_kill() {
	if [ "$2" != "" ]; then
		PREFIX="[${2}]"                
	else
		PREFIX=
	fi

	PID=`cat /var/run/$1.pid`

	# try kill(SIGTERM)
	COUNT=1
	PID_FIND=
	while [ $COUNT -lt 4 ]
	do
		if [ "`uname`" = "HP-UX" ]; then
			PID_FIND=`ps -p $PID | grep $PID`
		else
			PID_FIND=`ps -e | grep -w $PID`
		fi

		if [ -z "$PID_FIND" ]; then
			break
		fi

		print_log "${PREFIX}[try_kill][${COUNT}] trying to kill ${1}(${PID})"
		kill -15 $PID 2> /dev/null
		sleep 1

		COUNT=`expr $COUNT + 1`
	done

	# kill force(SIGKILL)
	if [ "`uname`" = "HP-UX" ]; then
		PID_FIND=`ps -p $PID | grep $PID`
	else
		PID_FIND=`ps -e | grep -w $PID`
	fi

	if [ -n "$PID_FIND" ]; then
		print_log "${PREFIX}[try_kill] killing ${1}(${PID})"
		kill -9 $PID 2> /dev/null
	fi
}

sleep 5

while [ 1 ]
do
	if [ "$APP_NAME" = "apc-agentd" ] || [ "$APP_NAME" = "epp-agentd" ]; then
		APC_AGENT_BINARY="`cat ./conf/v3dld.conf | grep ApcAgentBinary | cut -c 16-`"
        if [ "_$APC_AGENT_BINARY" = "_" ]; then
            APC_AGENT_BINARY="apc-agentd"
        else
            if [ "$APC_AGENT_BINARY" != "apc-agentd" ] && [ "$APC_AGENT_BINARY" != "epp-agentd" ]; then
                APC_AGENT_BINARY="apc-agentd"
            fi
        fi

		if [ "$APP_NAME" != "$APC_AGENT_BINARY" ]; then
			if [ -f /var/run/$APP_NAME.pid ]; then
				try_kill $APP_NAME "main"
			fi
			delete_file_check_file /var/run/$APP_NAME.pid

			cd ${INSTALL_PATH}
			print_log "staring ${APC_AGENT_BINARY}"
			./bin/$APC_AGENT_BINARY -f ./conf/v3dld.conf
			
			print_log "staring ${APC_AGENT_BINARY} watchdog"
			./bin/watchdog.sh $APC_AGENT_BINARY &
			
			delete_file_check_file /var/run/$APP_NAME.dog.pid

			check_process ${APC_AGENT_BINARY}
			exit
		fi
	fi

	ALL_PIDS=`ps -ef|grep /bin/$APP_NAME|grep -v watchdog.sh|grep -v grep|grep -v tail|grep -v cat|awk '{{ print $2 }}' `
	MAIN_PID=
	FIND=0

	if [ -f /var/run/$APP_NAME.pid ]; then
		#TARGET PROCESS PID
		MAIN_PID=`cat /var/run/$APP_NAME.pid`
	fi

	# TARGET PROCESS IS RUNNING
	if [ "_$MAIN_PID" != "_" ]; then
		for PID in $ALL_PIDS
		do  
			# CONFIRM TARGET PROCESS IS ALIVE
			if [ "$MAIN_PID" = "$PID" ]; then
				FIND=1
				continue
			fi

			error_log "KILL DUPLICATED TARGET PROCESS"
			error_log "  |--MAIN PORCESS: ${APP_NAME}($MAIN_PID)"
			error_log "  |--DUPLICATED PORCESS: ${APP_NAME}($PID)"
			kill -9 $PID
		done
	fi

	if [ $FIND -ne 1 ]; then
		cd ${INSTALL_PATH}

		if [ "_$MAIN_PID" != "_" ]; then
			error_log "TARGET PROCESS [${RED}${APP_NAME}${NC}] IS DOWN, but PID [${RED}${MAIN_PID}${NC}] FILE EXIST"
		else
			error_log "TARGET PROCESS [${RED}${APP_NAME}${NC}] IS DOWN"
		fi
		print_log "RE-START [${GR}${APP_NAME}${NC}]"
		if [ "_$APP_NAME" = "_v3fbmond" ]; then
		    ./bin/$APP_NAME -f ./conf/$CONF_FILE &
		else
		    ./bin/$APP_NAME -f ./conf/$CONF_FILE
		fi

		check_process ${APP_NAME}
	fi

	sleep 10
done
