#!/bin/sh

# ******************************************************************************
#
#  (C) 2018 AhnLab, Inc. All rights reserved.
#  Any part of this source code can not be copied with any method without
#  prior written permission from the author or authorized person.
#
# ******************************************************************************

export PWD PORT HOSTNAME OS BUILD_INFO BUILD VERSION IP SSL_LINE DISTRIB_ID

# =============================================================================================
# substitute string by using PERL or SED command
Replace()
{
	# PERL Replacing
    if [ $PERL_USE = "Y" ] && [ -z "$3" ]; then
		perl -pi -e "{$1}" $2
	# PERL Insertion
	elif [ $PERL_USE = "Y" ] && [ ! -z "$3" ]; then
		# check whether pattern already inserted or not
		IS_INSERTED=`grep -n "$1" $2 | awk -F: {'print $1'}`
		if [ -z "$IS_INSERTED" ]; then
			perl -pi -e "$.==$3 and print \"${1}\n\"" $2
		fi

	# SED Replacing
    elif [ $PERL_USE = "N" ] && [ -z "$3" ]; then
		  sed "${1}" $2 > $2.tmp && cat $2.tmp > $2 && rm $2.tmp	
	# SED Insertion
	elif [ $PERL_USE = "N" ] && [ ! -z "$3" ]; then
		# check whether pattern already inserted or not
		IS_INSERTED=`grep -n "$1" $2 | awk -F: {'print $1'}`
		if [ -z "$IS_INSERTED" ]; then
			#echo "INSERT Linux"
			sed "$3 i\\$1" $2 > $2.tmp && cat $2.tmp > $2 && rm $2.tmp
      	fi
    fi
}


# =============================================================================================
# check whether PERL package installed or not
func_check_perl()
{
	which perl > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		PERL_USE="Y"
	else
		PERL_USE="N"
	fi
}


# =============================================================================================
# check a OS is 'Gooroom 2.0' or higher
func_check_gooroom()
{
    if [ -f /etc/lsb-release ]; then
        DISTRIB_ID=`cat /etc/lsb-release | awk -F'=' '/DISTRIB_ID/ {print $2}'`
        print_log "[lsb-release] $DISTRIB_ID"
		DISTRIB_RELEASE=`cat /etc/lsb-release | awk -F'=' '/DISTRIB_RELEASE/ {print $2}'`
		print_log "[lsb-release] $DISTRIB_RELEASE"

		MAJOR_VER=`echo $DISTRIB_RELEASE | cut -d'.' -f1`
		MINOR_VER=`echo $DISTRIB_RELEASE | cut -d'.' -f2`

		if [ $DISTRIB_ID = "Gooroom" ] && [ $MAJOR_VER -ge 2 ]; then
			print_log "func_check_gooroom true"
			echo "true"
		else
			print_log "func_check_gooroom false"
			echo "false"
		fi
	fi
}


# =============================================================================================
# Port validation
func_check_port()
{
	ISNUMBER=`func_check_number $1`

	if [ $ISNUMBER -eq 1 ]; then
		if [ $1 -gt 0 -a $1 -lt 65536 ]; then
			echo 1
		else
			echo 0
		fi
	else
		echo 0
	fi
}


# =============================================================================================
GetUnusedPort()
{
	# 파일이 없을 경우 고려
	LOWER=`cat /proc/sys/net/ipv4/ip_local_port_range | awk -F' ' '{print $1}'`
	UPPER=`cat /proc/sys/net/ipv4/ip_local_port_range | awk -F' ' '{print $2}'`
	
	print_log "${PREFIX}    |--[GetUnusedPort] RANGE: ${LOWER}~${UPPER}"
	TRY=0
	while :
		do
			# 10개 
			PORT="`shuf -i ${LOWER}-${UPPER} -n 1`"
			print_log "${PREFIX}    |--[GetUnusedPort] CANDIATE PORT? ${PORT} (TRY $TRY)"
			ss -lpn | grep -q ":$PORT " || break

			TRY=`expr $TRY + 1`
			if [ $TRY -eq 10 ]; then
				break
			fi
	done

	if [ $TRY -eq 10 ]; then
		error_log "${PREFIX} [GetUnusedPort] GET NEW PORT FAIL"
		echo -1
		return
	else
		print_log "${PREFIX}    |--[GetUnusedPort] NEW TARGET PORT: [${PORT}]"
		echo $PORT
		return
	fi
}

CheckOS()
{
	DISTRIB=""
    if [ -f /etc/gooroom/info ]; then
        DISTRIB=`cat /etc/gooroom/info | awk -F= '$1 == "NAME"{print$2}' | sed -e 's/"//g'`
    fi
    
    if [ "${DISTRIB}" != "TmaxGooroom" ]; then
        DISTRIB=`lsb_release --id --short`
    fi
	
	echo $DISTRIB
	return
}

# =============================================================================================
# Check lighttpd port collision
CheckPort()
{
	if [ "$1" != "" ]; then
		PREFIX="|${1}|"
	else
		PREFIX=
	fi

	CHECK_PORT=`cat ./conf/lighttpd.conf | grep server.port | tr -d ' ' | awk -F= '{print $2}'`
	if [ "_$CHECK_PORT" = "_" ]; then
		error_log "${PREFIX} [CheckPort] SERVER PORT IS UNKNOWN"
		echo 1
		return
	fi

	ORIGIN_PORT=$CHECK_PORT
	COLLISION_PROCESS=`lsof -Pi :${CHECK_PORT} -sTCP:LISTEN | tail -n 1 | awk '{print $1}'`

	if [ ! -z $COLLISION_PROCESS ]; then
		error_log "${PREFIX} [CheckPort] SERVER PORT [${CHECK_PORT}] ALREADY IN USED BY [${COLLISION_PROCESS}]"
		CHECK_PORT=`GetUnusedPort`
	fi

	# copy web_error.jpg file to '/etc' and link desktop icon
	if [ $CHECK_PORT -eq -1 ]; then
		cp $INSTALL_PATH/www/img/web_error.jpg /etc/
		
		DISTRIB=`CheckOS`
		if [ $DISTRIB = "Gooroom" ]; then
			Replace "s/^Exec=.*$/Exec=gooroom-browser file:\/\/\/etc\/web_error.jpg/g" /usr/share/applications/$MODE2.desktop
		elif [ $DISTRIB = "TmaxGooroom" ]; then
			Replace "s/^Exec=.*$/Exec=togate-browser file:\/\/\/etc\/web_error.jpg/g" /usr/share/applications/$MODE2.desktop
		else
			Replace "s/^Exec=.*$/Exec=firefox file:\/\/\/etc\/web_error.jpg/g" /usr/share/applications/$MODE2.desktop
		fi
		echo 1
		return
	else
		# web port is changed
		if [ $CHECK_PORT != $ORIGIN_PORT ]; then
			Replace "s/^server.port.*$/server.port = $CHECK_PORT/g" ./conf/lighttpd.conf
			print_log "${PREFIX}    |--LIGHTTPD SERVER PORT CHANGED TO [${CHECK_PORT}]"
		fi

		DISTRIB=`CheckOS`
		if [ $DISTRIB = "Gooroom" ]; then
			Replace "s/^Exec=.*$/Exec=gooroom-browser http:\/\/localhost:$CHECK_PORT/g" /usr/share/applications/$MODE2.desktop
		elif [ $DISTRIB = "TmaxGooroom" ]; then
			Replace "s/^Exec=.*$/Exec=togate-browser http:\/\/localhost:$CHECK_PORT/g" /usr/share/applications/$MODE2.desktop
		else
			Replace "s/^Exec=.*$/Exec=firefox http:\/\/localhost:$CHECK_PORT/g" /usr/share/applications/$MODE2.desktop
		fi
		echo 0
		return
	fi
}


# =============================================================================================
# Distribute ID
CheckDistributeID()
{
	if [ "$1" != "" ]; then
		PREFIX="|${1}|"
	else
		PREFIX=
	fi

    if which lsb_release > /dev/null 2>&1
    then
        DISTRIB_ID=`lsb_release --id --short`
        print_log "${PREFIX} [lsb_release] ID: $DISTRIB_ID"
    elif [ -f /etc/os-release ]
    then
        DISTRIB_ID=`cat /etc/os-release | awk -F'=' '/PRETTY/ {print $2}' | sed -e 's/"//g' | sed -e 's/ //g'`
		print_log "${PREFIX} [os-release] ID: $DISTRIB_ID"
    elif [ -f /etc/redhat-release ]
    then
        DISTRIB_ID=`cat /etc/redhat-release | sed -e 's/ //g'`
		print_log "${PREFIX} [redhat-release] ID: $DISTRIB_ID"
    else
        DISTRIB_ID="unknown"
        error_log "${PREFIX} DISTRIB ID UNKNOWN"
    fi  

    case $DISTRIB_ID in
        Debian* )
            DISTRIB_ID="Debian"
            ;;
        Ubuntu* )
            DISTRIB_ID="Ubuntu"
            ;;
        Fedora* )
            DISTRIB_ID="Fedora"
            ;;
        RedHatEnter* )
            DISTRIB_ID="RHEL"
            ;;
        EnterpriseEnterpriseServer | Oracle* )
            DISTRIB_ID="OracleLinux"
            ;;
        RedHatLinux* )
            DISTRIB_ID="RedHat"
            ;;
        openSUSE* )
            DISTRIB_ID="openSUSE"
            ;;
        RedHatAdvancedServer | CentOS* )
            DISTRIB_ID="CentOS"
            ;;
		SUSE* )
			DISTRIB_ID="SUSE"
			;;
		ProLinux* )
			DISTRIB_ID="Tmaxos"
			;;
        Gooroom* )
			DISTRIB_ID="Gooroom"
			;;	
    esac    

	if ! grep DistribID $INSTALL_PATH/conf/v3dld.conf >/dev/null ; then
		echo "DistribID=${DISTRIB_ID}" >> $INSTALL_PATH/conf/v3dld.conf
	else
		perl -pi -e 's/^DistribID.*$/DistribID=$ENV{DISTRIB_ID}/g' $INSTALL_PATH/conf/v3dld.conf
	fi
}


# =============================================================================================
# BASE SETTINGS

# COLORS
RED='\033[1;31m'
BLUE='\033[1;34m'
GR='\033[1;32m'
NC='\033[0m'

# VARS
UNAME=`uname`
USERID=""
HOSTNAME=`hostname`
BUILD=0
OLD_BUILD=0
LIGHTTPD_PID="/var/run/v3dld_web.pid"
INSTALL_PATH="`cat /etc/v3dl.install.conf | grep InstallPath | cut -c 13-`"
SU_PID="`ps -ef | grep $INSTALL_PATH/suarez/bin/suarez | grep -v grep | grep -v 'sh -c' | awk '{{ print $2 }}'`"
CWD=$(dirname ${0} | sed 's/\//\\\//g')
PERL_USE="N"

func_check_perl

# PRODUCT MODE SET
PRODUCT=`cat ${INSTALL_PATH}/conf/v3dld.conf | grep ProductName | cut -c 13-`
if [ "$PRODUCT" = "V3_DESKTOP_FOR_LINUX" ]; then
	MODE="V3DL"
	MODE2="v3desktop"
elif [ "$PRODUCT" = "V3LITE_OPENOS" ]; then
	MODE="V3LL"
	MODE2="v3lite"
elif [ -z "$PRODUCT" ]; then
	# BEFORE VER 1.0.9.1
	Replace 'ProductName=V3_DESKTOP_FOR_LINUX' ${INSTALL_PATH}/conf/v3dld.conf 1
else
	exit
fi

# includes util.sh to log
export LOG_FILE_NAME WATCHDOG_LOG_FILE_NAME
LOG_FILE_NAME=`basename ${0}`
WATCHDOG_LOG_FILE_NAME="watchdog.sh"

. ${INSTALL_PATH}/bin/util.sh

# check root permission/user
if [ -f /usr/xpg4/bin/id ]; then
	USERID=`/usr/xpg4/bin/id -u`
else
	USERID=`/usr/bin/id -u`
fi

if  [ $USERID != '0' ]; then
	echo "[WARN] only root user can execute V3"
	exit
fi

# get pwd
PWD=""
if [ "`echo ${0}|cut -b 1`" = '/' ]; then
	PWD=`dirname ${0}`
else
	PWD=`pwd`
fi
cd ${PWD}

# check /var/run dir
if [ ! -d /var/run ]; then
	print_log "[main] creating /var/run directory"
	mkdir /var/run
fi

# get OS, Distrib version
OS=""
UNAME=`uname`

case "$UNAME" in
	HP-UX)
	;;
	SunOS)
		if test `uname -i` = "i86pc"; then
			test
		else
			test
		fi
	;;
	FreeBSD)
	;;
	Linux)
	;;
	AIX)
		OS="aix"
	;;
esac

# APC/EPP SETTING (Only V3DL)
APC_USE=`cat ./conf/v3dld.conf | grep ApcUse | cut -c 8-`
APC_AGENT_BINARY="`cat ./conf/v3dld.conf | grep ApcAgentBinary | cut -c 16-`"
if [ "_$APC_AGENT_BINARY" = "_" ]; then
    APC_AGENT_BINARY="apc-agentd"
else
    if [ "$APC_AGENT_BINARY" != "apc-agentd" ] && [ "$APC_AGENT_BINARY" != "epp-agentd" ] && [ "$APC_AGENT_BINARY" != "gpms-agentd" ]; then
        error_log "[main] UNKNONW APC BINARY [$APC_AGENT_BINARY]"
        APC_AGENT_BINARY="apc-agentd"
    fi
fi

# =============================================================================================
# terminating a process with SIGKILL signal
SIGKILL() {
	if [ "$2" != "" ]; then
		PREFIX="|${2}|"                
	else
		PREFIX=
	fi

	if [ "${1}" = "lighttpd" ]; then
		PID_FILE=$LIGHTTPD_PID
	else
		PID_FILE="/var/run/${1}.pid"
	fi

	if [ -f $PID_FILE ]; then
		PID=`cat ${PID_FILE}`
	else
		error_log "${PREFIX} [SIGKILL] '${1}' PID_FILE(${PID_FILE}) NOT FOUND"
		return 1
	fi
	
	print_log "${PREFIX} [SIGKILL] KILL ${1}(${PID})"
	kill -9 ${PID} 2> /dev/null
}


# =============================================================================================
# see if a process is running or not
check_process() {
	if [ "$2" != "" ]; then
		PREFIX="|${2}|"
	else
		PREFIX=
	fi
	
	if [ "${1}" = "lighttpd" ]; then
		PID_FILE=$LIGHTTPD_PID
		WATCHDOG=
	else
		PID_FILE="/var/run/${1}.pid"
		WATCHDOG="/var/run/${1}.dog.pid"
	fi

	PID_FIND=
	if [ -f $PID_FILE ]; then
		PID=`cat ${PID_FILE}`
		if [ "${PID}" != "" ]; then
			if [ "`uname`" = "HP-UX" ]; then
				PID_FIND=`ps -p ${PID} | grep ${PID}`
			else
				PID_FIND=`ps -e | grep -w ${PID}`
			fi
		else
			PID_FIND=
		fi

		# running
		if [ "_" != "_${PID_FIND}" ]; then
			if [ "$2" = "start" ] || [ "$2" = "lighttpdstart" ] || [ "$2" = "v3dldstart" ]; then
				print_log "${PREFIX} [check_process] ${1}(${PID}) RUNNING"
			else
				error_log "${PREFIX} [check_process] ${1}(${PID}) RUNNING"
			fi
		# not running
		else
			if [ "$2" = "start" ] || [ "$2" = "lighttpdstart" ] || [ "$2" = "v3dldstart" ]; then
				error_log "${PREFIX} [check_process] ${1}(${PID}) NOT RUNNING"
			else
				print_log "${PREFIX} [check_process] ${1}(${PID}) NOT RUNNING"
			fi
		fi
	else
		PID=`ps -ef|grep /bin/${1}|grep -v watchdog.sh|grep -v grep|grep -v tail|grep -v cat|awk '{{ print $2 }}' `
		if [ "" != "${PID}" ]; then
			if [ "$2" = "start" ] || [ "$2" = "lighttpdstart" ] || [ "$2" = "v3dldstart" ]; then
				print_log "${PREFIX} [check_process] ${1}(${PID}) RUNNING"
			else
				error_log "${PREFIX} [check_process] ${1}(${PID}) RUNNING"
			fi
		else
			if [ "$2" = "start" ] || [ "$2" = "lighttpdstart" ] || [ "$2" = "v3dldstart" ]; then
				error_log "${PREFIX} [check_process] ${1} NOT RUNNING"
			else
				print_log "${PREFIX} [check_process] ${1} NOT RUNNING"
			fi
		fi
	fi
}


# =============================================================================================
# delete file  and check_file
delete_file() {
	if [ "$2" != "" ]; then
		PREFIX="|${2}|"                
	else
		PREFIX=
	fi

	if [ "$1" != "" ]; then
		FILE="${1}"
	else
		error_log "${PREFIX}    |--[delete_file] TARGET FILE NOT SPECIFIED"
		return 1
	fi

	if [ -f $FILE ]; then
		print_log "${PREFIX}    |--[delete_file] DELETE TARGET FILE '${FILE}'"
		rm -rf "${FILE}"
	else
		print_log "${PREFIX}    |--[delete_file] TARGET FILE NOT FOUND : '${FILE}'"
	fi
}


# =============================================================================================
# see if a file is exist or not
check_file() {
	if [ "$2" != "" ]; then
		PREFIX="|${2}|"
	else
		PREFIX=
	fi
	
	if [ "$1" != "" ]; then
		FILE="${1}"
	else
		error_log "$PREFIX    |--[check_file] TARGET FILE NOT SPECIFIED"
		return 1
	fi

	if [ -f $FILE ]; then
		error_log "$PREFIX    |--[check_file] TARGET FILE '${FILE}' STILL EXIST"
	else
		print_log "$PREFIX    |--[check_file] TARGET FILE '${FILE}' DELETE CONFIRM"
	fi
}


# =============================================================================================
# call delete_file and then check_file
delete_file_check_file() {
	delete_file "${1}" "${2}"
 	check_file "${1}" "${2}"
}


# =============================================================================================
# see if a process is running or not
print_status() {
	if [ "${1}" = "lighttpd" ]; then
		PID_FILE=$LIGHTTPD_PID
		WATCHDOG=
	else
		PID_FILE="/var/run/${1}.pid"
		WATCHDOG="/var/run/${1}.dog.pid"
	fi

	PID_FIND=
	if [ -f $PID_FILE ]; then
		PID=`cat ${PID_FILE}`
		if [ "${PID}" != "" ]; then
			if [ "`uname`" = "HP-UX" ]; then
				PID_FIND=`ps -p ${PID} | grep ${PID}`
			else
				PID_FIND=`ps -e | grep -w ${PID}`
			fi
		else
			PID=`ps -ef|grep /bin/${1}|grep -v watchdog.sh|grep -v grep|grep -v tail|grep -v cat|awk '{{ print $2 }}' `
			if [ "" != "${PID}" ]; then
				PID_FIND=PID
			else
				PID_FIND=
			fi
		fi
	else
		PID=`ps -ef|grep /bin/${1}|grep -v watchdog.sh|grep -v grep|grep -v tail|grep -v cat|awk '{{ print $2 }}' `
		if [ "" != "${PID}" ]; then
			PID_FIND=PID
		else
			PID_FIND=
		fi
	fi

	if [ "_" != "_${PID_FIND}" ]; then
		echo "${GR}RUNNING${NC} (${PID})"
	else
		echo "${RED}NOT RUNNING${NC}"
	fi
}


# =============================================================================================
# terminating process with SIGTERM first, if not SIGKILL
SIGTERM() {
	if [ "$2" != "" ]; then
		PREFIX="|${2}|"           
	else
		PREFIX=
	fi

	PID=`cat /var/run/$1.pid`

	# try kill(SIGTERM)
	print_log "${PREFIX} [SIGTERM] TERMINATE ${1}(${PID})"
	COUNT=1
	PID_FIND=
	while [ $COUNT -lt 4 ]
	do
		if [ "`uname`" = "HP-UX" ]; then
			PID_FIND=`ps -p $PID | grep $PID`
		else
			PID_FIND=`ps -e | grep -w $PID`
		fi

		if [ -z "$PID_FIND" ]; then
			break
		fi

		print_log "${PREFIX}    |--TRY SEQ[${COUNT}]"
		kill -15 $PID 2> /dev/null
		sleep 1

		COUNT=`expr $COUNT + 1`
	done

	# kill force(SIGKILL)
	if [ "`uname`" = "HP-UX" ]; then
		PID_FIND=`ps -p $PID | grep $PID`
	else
		PID_FIND=`ps -e | grep -w $PID`
	fi

	if [ -n "$PID_FIND" ]; then
		error_log "${PREFIX} [SIGTERM] TERMINATE ${1}(${PID}) FAIL"
		print_log "${PREFIX}    |--SIGKILL ${1}(${PID})"
		kill -9 $PID 2> /dev/null
	fi
}


# =============================================================================================
# Get linux kernel version(linux only)
getKVC()
{
	VERSION=`echo $1 | awk -F'.' '{print $1}'`
	MAJOR=`echo $1 | awk -F'.' '{print $2}'`
	MINOR=`echo $1 | awk -F'.' '{print $3}'`
	echo `expr \( $VERSION \* 65536 \) + \( $MAJOR \* 256 \) + $MINOR`
}


# =============================================================================================
# Get libc version(linux only)
getGlibcVer()
{
	VERSION=`echo $1 | awk -F'.' '{print $1}'`
	MAJOR=`echo $1 | awk -F'.' '{print $2}'`
	echo `expr \( $VERSION \* 256 \) + $MAJOR`
}


# =============================================================================================
# Get kernel config(linux only)
getKernelConfig()
{
	KERNEL_RELEASE=`uname -r`
	if [ -f "/boot/config-${KERNEL_RELEASE}" ]; then
		KERNEL_CONFIG=`cat /boot/config-$KERNEL_RELEASE | grep -w $1 | awk -F'=' '{print $2}'`

		if [ -n "$KERNEL_CONFIG" ]; then
			echo "$KERNEL_CONFIG"
		else
			echo "n"
		fi
	else
		error_log "[getKernelConfig] config file error : /boot/config-${KERNEL_RELEASE}"
		echo "n"
	fi
}


# =============================================================================================
# check which notify mechanism is supported
CheckNotifySupport()
{
	KERNEL_VER=`uname -r | awk -F'-' '{print $1}' | awk -F'.' '{print $1"."$2"."$3}'`
	KERNEL_INOTIFY_VC=`getKVC 2.6.13`
	KERNEL_FANOTIFY_VC=`getKVC 2.6.37`
	KERNEL_CURRENT_VC=`getKVC $KERNEL_VER`

	LIBC_PATH=`ldd /bin/sh | grep libc.so | head -n1 | awk -F'=> ' '{print $2}' | awk -F' ' '{print $1}'`
	LIBC_VERSION=`echo $($LIBC_PATH) | awk -F'version' '{print $2}' | awk -F' ' '{print $1}' | sed 's/,//'`
	LIBC_INOTIFY_VC=`getGlibcVer 2.4`
	LIBC_FANOTIFY_VC=`getGlibcVer 2.13`
	LIBC_CURRENT_VC=`getGlibcVer $LIBC_VERSION`

	CONFIG_FANOTIFY=`getKernelConfig CONFIG_FANOTIFY`
	# CONFIG_FANOTIFY_ACCESS_PERMISSIONS=`getKernelConfig CONFIG_FANOTIFY_ACCESS_PERMISSIONS`
	CONFIG_INOTIFY=`getKernelConfig CONFIG_INOTIFY`
	CONFIG_INOTIFY_USER=`getKernelConfig CONFIG_INOTIFY_USER`

	if [ $KERNEL_CURRENT_VC -ge $KERNEL_FANOTIFY_VC -a $LIBC_CURRENT_VC -ge $LIBC_FANOTIFY_VC ]; then
		if [ "$CONFIG_FANOTIFY" = "y" -o "$CONFIG_FANOTIFY" = "Y" ]; then
			echo "fanotifybase"
		fi
	else
		if [ $KERNEL_CURRENT_VC -ge $KERNEL_INOTIFY_VC -a $LIBC_CURRENT_VC -ge $LIBC_INOTIFY_VC ]; then
			if [ "$CONFIG_INOTIFY" = "y" -o "$CONFIG_INOTIFY" = "Y" -o "$CONFIG_INOTIFY_USER" = "y" -o "$CONFIG_INOTIFY_USER" = "Y" ]; then
				echo "inotifybase"
			fi
		fi
	fi
}


# =============================================================================================
# make(call) watchdog process
callWatchdog()
{
    nohup ./bin/watchdog.sh $1 $INSTALL_PATH v3dld.conf > /dev/null 2>&1 &
}


# =============================================================================================
case "${1}" in
	"status")
		if [ "$MODE" = "V3DL" ]; then
 			printf "${BLUE}"
			printf " _    _______    ____  ___________ __ ____________  ____  \n"
			printf "| |  / /__  /   / __ \/ ____/ ___// //_/_  __/ __ \/ __ \ \n"
			printf "| | / / /_ <   / / / / __/  \__ \/ ,<   / / / / / / /_/ / \n"
			printf "| |/ /___/ /  / /_/ / /___ ___/ / /| | / / / /_/ / ____/  \n"
			printf "|___//____/  /_____/_____//____/_/ |_|/_/  \____/_/       \n"
			printf "                                               for Linux\n${NC}"

		
		elif [ "$MODE" = "V3LL" ]; then
			printf "     ${BLUE} _    _______    __    _ __ \n"
			printf "     | |  / /__  /   / /   (_) /____ \n"
			printf "     | | / / /_ <   / /   / / __/ _ \\ \n"
			printf "     | |/ /___/ /  / /___/ / /_/  __/ \n"
			printf "     |___//____/  /_____/_/\__/\___/ \n"
			printf "                     For OpenSourceOS\n${NC}"
		fi

		printf " +--------------------------------------+\n"
		printf " |            PROCESS STATUS            |\n"
		printf " +--------------------------------------+\n"

		printf " v3dld                  - "
		print_status v3dld

		printf " v3dld watchdog         - "
		print_status v3dld.dog

		printf " v3dl-agentd            - "
		print_status v3dl-agentd

		printf " v3dl-agentd watchdog   - "
		print_status v3dl-agentd.dog

		printf " lighttpd               - "
		print_status lighttpd

		if [ "$APC_USE" != "No" ]; then
			if [ "$APC_AGENT_BINARY" = "gpms-agentd" ]; then
				if [ "`func_check_gooroom`" = "true" ]; then
					printf " $APC_AGENT_BINARY            - "
					print_status $APC_AGENT_BINARY

					printf " $APC_AGENT_BINARY watchdog   - "
					print_status $APC_AGENT_BINARY.dog
				fi
			else
				printf " $APC_AGENT_BINARY            - "
				print_status $APC_AGENT_BINARY

				printf " $APC_AGENT_BINARY watchdog   - "
				print_status $APC_AGENT_BINARY.dog
			fi

		fi

		if [ "`uname`" = "Linux" ]; then
			if [ "`CheckNotifySupport`" = "fanotifybase" ]; then
				printf " v3fbmond               - "
				print_status v3fbmond

				printf " v3fbmond watchdog      - "
				print_status v3fbmond.dog
			fi
		fi

		printf " ----------------------------------------\n"
	;;

	"start")
		print_log "============================================================"
		print_watchdog_log "==================================================="
		print_log "|start| STARTING V3"
		print_watchdog_log "V3 STARTED"

		# when APC/EPP Serial is changed
		./bin/migration.sh		

		#echo "STARTING V3"
		Replace "s/^ServerName.*$/ServerName=$HOSTNAME/g" ./conf/v3dld.conf

		if [ -f ./system/bldinfo.ini ]; then
			BUILD=`cat ./system/bldinfo.ini | grep BuildNumber | awk -F'.' '{ print $4 }'`
			VERSION=`cat ./system/bldinfo.ini | grep Version | awk -F'=' '{ print $2 }'`
			OLD_BUILD=`cat ./conf/v3dld.conf | grep Build | awk -F'=' '{ print $2 }'`

			if  [ $BUILD -gt $OLD_BUILD ]; then
				Replace "s/^Version.*$/Version=${VERSION}/g" ./conf/v3dld.conf
				Replace "s/^Build.*$/Build=${BUILD}/g" ./conf/v3dld.conf
			fi
		fi

		CheckDistributeID start

		# v3dld
		print_log "|start| STARTING v3dld"
		if [ -f /var/run/v3dld.pid ]; then
			print_log "|start| v3dld ALREADY RUNNING or PID FILE EXIST"
			SIGTERM v3dld "start"
		fi
		./bin/v3dld -f ./conf/v3dld.conf


		# lighttpd
		if [ -f "$LIGHTTPD_PID" ]; then
			SIGKILL lighttpd "start"
		fi

		RET=`CheckPort "start"`
		if [ $RET -eq 0 ]; then
			print_log "|start| STARTING lighttpd"
			./bin/lighttpd -f ./conf/lighttpd.conf -m ./lib
		else
			error_log "|start| lighttpd PORT SETTING FAIL($RET)"
			


			# v3lite 를 종료하진 않음
			# SIGTERM v3dld "start"
		fi

		# v3dl-agentd
		print_log "|start| STARTING v3dl-agentd"
		if [ -f /var/run/v3dl-agentd.pid ]; then
			SIGKILL v3dl-agentd "start"
		fi
		print_log "|start| v3dl-agentd ALREADY RUNNING or PID FILE EXIST"
		./bin/v3dl-agentd -f ./conf/v3dld.conf

		
		# v3filemond
		V3FILEMOND=                
		if [ "`uname`" = "Linux" ]; then
			if [ "`CheckNotifySupport`" = "fanotifybase" ]; then
				print_log "[start] fanotify supported"
				V3FILEMOND=v3fbmond
			fi

			if [ "_$V3FILEMOND" != "_" ]; then
				if [ -f /var/run/$V3FILEMOND.pid ]; then
					print_log "|start| v3fbmond ALREADY RUNNING or PID FILE EXIST"
					SIGTERM $V3FILEMOND "start"
				fi
				./bin/$V3FILEMOND &
			fi
		fi

		# APC_AGENT_BINARY
		if [ "$APC_USE" != "No" ]; then
		    # apc-agentd / epp-agentd / gpms-agentd
			if [ -f /var/run/apc-agentd.pid ]; then
				print_log "|start| apc-agentd ALREADY RUNNING or PID FILE EXIST"
				SIGKILL apc-agentd "start"
			fi

			if [ -f /var/run/epp-agentd.pid ]; then
				print_log "|start| epp-agentd ALREADY RUNNING or PID FILE EXIST"
				SIGTERM epp-agentd "start"
			fi

			if [ -f /var/run/gpms-agentd.pid ]; then
				if [ "`func_check_gooroom`" = "true" ]; then
					print_log "|start| gpms-agentd ALREADY RUNNING or PID FILE EXIST"
					SIGTERM gpms-agentd "start"
				fi
			fi

			print_log "|start| STARTING ${APC_AGENT_BINARY}"
			if [ "$APC_AGENT_BINARY" = "gpms-agentd" ]; then
				if [ "`func_check_gooroom`" = "true" ]; then
					./bin/$APC_AGENT_BINARY &
				fi
			else 
				./bin/$APC_AGENT_BINARY -f ./conf/v3dld.conf
			fi
			
			# watchdog - $APC_AGENT_BINARY
			if [ -f /var/run/$APC_AGENT_BINARY.dog.pid ]; then
				SIGKILL $APC_AGENT_BINARY.dog "start"
			fi
			if [ "$APC_AGENT_BINARY" = "gpms-agentd" ]; then
				if [ "`func_check_gooroom`" = "true" ]; then
					print_log "|start| STARTING ${APC_AGENT_BINARY} watchdog"
					callWatchdog $APC_AGENT_BINARY
				fi
			else
				print_log "|start| STARTING [${APC_AGENT_BINARY}] watchdog"
				callWatchdog $APC_AGENT_BINARY
			fi
		fi

		# watchdog - v3dld
		print_log "|start| STARTING v3dld watchdog"
		if [ -f /var/run/v3dld.dog.pid ]; then
			print_log "|start| v3dld watchdog ALREADY RUNNING or PID FILE EXIST"
			SIGKILL v3dld.dog "start"
		fi
		callWatchdog v3dld

		# watchdog - v3dl-agentd
		if [ -f /var/run/v3dl-agentd.dog.pid ]; then
			print_log "|start| v3dl-agent.dog watchdog ALREADY RUNNING or PID FILE EXIST"
			SIGKILL v3dl-agentd.dog "start"
		fi
		callWatchdog v3dl-agentd

		# watchdog - v3filemond
		if [ "`uname`" = "Linux" ]; then
			if [ "`CheckNotifySupport`" = "fanotifybase" ]; then
				V3FILEMOND=v3fbmond
				print_log "|start| STARTING ${V3FILEMOND} watchdog"
			fi

			if [ "_$V3FILEMOND" != "_" ]; then
				if [ -f /var/run/$V3FILEMOND.dog.pid ]; then
					print_log "|start| $V3FILEMOND watchdog ALREADY RUNNING or PID FILE EXIST"
					SIGKILL "${V3FILEMOND}.dog" "start"
				fi
				callWatchdog $V3FILEMOND
			fi
		fi

		print_log "|start| PATCH UPDATE RUNNING CHECK"
		delete_file_check_file "update/.__sub_info" "start"

		# check process is started
		check_process v3dld  "start"
		if [ "$APC_USE" != "No" ]; then
				check_process ${APC_AGENT_BINARY} "start"
		fi
		check_process "lighttpd"  "start"

		check_process v3dl-agentd  "start"
		
		if [ "_$V3FILEMOND" != "_" ]; then
				check_process ${V3FILEMOND}  "start"
		fi

		print_log "|start| STARTING V3 DONE\n"
		
	;;
	"stop")
		print_log "============================================================"
		print_log "|stop| STOP V3"
		#echo "terminating the process..."

		# watchdog - v3dld
		if [ -f /var/run/v3dld.dog.pid ]; then
			SIGKILL v3dld.dog "stop"
		fi
		delete_file_check_file "/var/run/v3dld.dog.pid" "stop"

		# watchdog - v3dl-agentd
		if [ -f /var/run/v3dl-agentd.dog.pid ]; then
			SIGKILL v3dl-agentd.dog "stop"
		fi
		delete_file_check_file "/var/run/v3dl-agentd.dog.pid" "stop"

		# watchdog - v3filemond
		if [ "`uname`" = "Linux" ]; then
			if [ "`CheckNotifySupport`" = "fanotifybase" ]; then
				V3FILEMOND=v3fbmond
			fi

			if [ "_$V3FILEMOND" != "_" ]; then
				if [ -f /var/run/$V3FILEMOND.dog.pid ]; then
                    SIGKILL v3fbmond.dog "stop"
				fi
                delete_file_check_file "/var/run/$V3FILEMOND.dog.pid" "stop"
			fi
		fi

		# lighttpd
		if [ -f "$LIGHTTPD_PID" ]; then
			SIGKILL lighttpd "stop"
		fi
		delete_file_check_file "$LIGHTTPD_PID" "stop"

		# v3dl-agentd
		if [ -f /var/run/v3dl-agentd.pid ]; then
			SIGKILL v3dl-agentd "stop"
		fi
		delete_file_check_file "/var/run/v3dl-agentd.pid" "stop"

		if [ "$APC_USE" != "No" ]; then
			# watchdog - apc-agentd
			if [ -f /var/run/apc-agentd.dog.pid ]; then
            			SIGKILL apc-agentd.dog "stop"
			fi
            delete_file_check_file "/var/run/apc-agentd.dog.pid" "stop"

			# apc-agentd
			if [ -f /var/run/apc-agentd.pid ]; then
            			SIGKILL apc-agentd "stop"
			fi
            delete_file_check_file "/var/run/apc-agentd.pid" "stop"

			# watchdog - epp-agentd
			if [ -f /var/run/epp-agentd.dog.pid ]; then
            			SIGKILL epp-agentd.dog "stop"
			fi
            delete_file_check_file "/var/run/epp-agentd.dog.pid" "stop"

			# epp-agentd
			if [ -f /var/run/epp-agentd.pid ]; then
				SIGTERM epp-agentd "stop"
			fi
            delete_file_check_file "/var/run/epp-agentd.pid" "stop"

			# watchdog - gpms-agentd
			if [ -f /var/run/gpms-agentd.dog.pid ]; then
				if [ "`func_check_gooroom`" = "true" ]; then	
					SIGKILL gpms-agentd.dog "stop"
				fi
				delete_file_check_file "/var/run/gpms-agentd.dog.pid" "stop"
			fi

			# gpms-agentd
			if [ -f /var/run/gpms-agentd.pid ]; then
				if [ "`func_check_gooroom`" = "true" ]; then
					SIGTERM gpms-agentd "stop"
				fi
				delete_file_check_file "/var/run/gpms-agentd.pid" "stop"
			fi
		fi

		# v3filemond
		if [ "`uname`" = "Linux" ]; then
			if [ "`CheckNotifySupport`" = "fanotifybase" ]; then
				V3FILEMOND=v3fbmond
			fi

			if [ "_$V3FILEMOND" != "_" ]; then
				if [ -f /var/run/$V3FILEMOND.pid ]; then
					SIGTERM $V3FILEMOND "stop"
				fi
            	delete_file_check_file "/var/run/$V3FILEMOND.pid" "stop"
			fi
		fi
		
		# v3dld
		if [ -f /var/run/v3dld.pid ]; then
			SIGTERM v3dld "stop"
		fi
		delete_file_check_file "/var/run/v3dld.pid" "stop"

		# suarez
		if [ "$SU_PID" != "" ]; then
			print_log "[stop] killing suarez"
			kill -15 $SU_PID 2> /dev/null
		fi

		# check process is stopped
		check_process v3dld  "stop"
		if [ "$APC_USE" != "No" ]; then
			check_process ${APC_AGENT_BINARY} "stop"
			if [ "$APC_AGENT_BINARY" = "gpms-agentd" ]; then
				check_process $GPMS_AGENT_BINARY "stop"
			fi
		fi

		check_process "lighttpd"  "stop"

		check_process v3dl-agentd  "stop"

		if [ "_$V3FILEMOND" != "_" ]; then
			check_process ${V3FILEMOND}  "stop"
		fi

		print_log "|stop| STOP V3 DONE\n"
		print_watchdog_log "V3 STOPPED"
	;;
	"lighttpdstart")
		print_log "============================================================"
		print_log "|lighttpdstart| STARTING LIGHTTPD (TARGET PROCESS)"
		# echo "Starting the lighttpd..."

		if [ "${2}" != "" ]; then
			print_log "|lighttpdstart| CHANGE SSL CONFIGURATIONS"
			# SSL Setting mirgration
			grep "^#ssl.engine" ./conf/lighttpd.conf > /dev/null
			if [ $? -eq 0 ]; then
				Replace "s/^#ssl.engine.*$/ssl.engine = \"disable\"/g" ./conf/lighttpd.conf

				print_log "|lighttpdstart|    |--ssl.pemfile path [\"$CWD\/conf\/lighttpd.pem\"]"
				Replace "s/^#ssl.pemfile.*$/ssl.pemfile = \"$CWD\/conf\/lighttpd.pem\"/g" ./conf/lighttpd.conf

				# get line number of SSL_LINE pattern
				SSL_LINE=`grep -n "^ssl.pemfile" ./conf/lighttpd.conf | awk -F: {'print $1'}`

				# insert ssl configuration content
				print_log "|lighttpdstart|    |--ssl.cipher-list changed"
				Replace 'ssl.cipher-list = \"TLSv1.2:!aNULL:!eNULL:!DSS;\"' ./conf/lighttpd.conf $SSL_LINE

				print_log "|lighttpdstart|    |--ssl.use-sslv3 disabled"
				Replace 'ssl.use-sslv3 = \"disable\"' ./conf/lighttpd.conf $SSL_LINE

				print_log "|lighttpdstart|    |--ssl.use-sslv2 disabled"
				Replace 'ssl.use-sslv2 = \"disable\"' ./conf/lighttpd.conf $SSL_LINE
			fi

			case "${2}" in
				"0")
					print_log "|lighttpdstart|    |--ssl.engine disabled"
					Replace "s/^ssl.engine.*$/ssl.engine = \"disable\"/g" ./conf/lighttpd.conf
				;;

				"1")
					print_log "|lighttpdstart|    |--ssl.engine enabled"
					Replace "s/^ssl.engine.*$/ssl.engine = \"enable\"/g" ./conf/lighttpd.conf
				;;

				*)
				;;
			esac
		fi

		if [ "${3}" != "" ] && [ "${3}" != "0" ]; then
			print_log "|lighttpdstart| CHANGE LIGHTTPD CONFIGURATIONS"
			print_log "|lighttpdstart|    |--server.port changed [${3}]"
			PORT=${3}
			Replace "s/^server.port.*$/server.port = $PORT/g" ./conf/lighttpd.conf
			
			DISTRIB=`CheckOS`
			if [ $DISTRIB = "Gooroom" ]; then
				Replace "s/^Exec=.*$/Exec=gooroom-browser http:\/\/localhost:$PORT/g" /usr/share/applications/$MODE2.desktop
				print_log "|lighttpdstart|    |--Distrib ID:$DISTRIB / Browser: gooroom-browser[$PORT]"
			elif [ $DISTRIB = "TmaxGooroom" ]; then
				Replace "s/^Exec=.*$/Exec=togate-browser http:\/\/localhost:$PORT/g" /usr/share/applications/$MODE2.desktop
				print_log "|lighttpdstart|    |--Distrib ID:$DISTRIB / Browser: togate-browser[$PORT]"
			else
				Replace "s/^Exec=.*$/Exec=firefox http:\/\/localhost:$PORT/g" /usr/share/applications/$MODE2.desktop
				print_log "|lighttpdstart|    |--Distrib ID:$DISTRIB / Browser: FireFox[$PORT]"
			fi
		fi 

		if [ "${4}" != "" ] && [ "${4}" != "0" ]; then
			IP=`echo "^${4}$" | sed -e 's/|/\$|\^/g'`
			
			# IP migration
			grep "^\$HTTP\[\"remoteip\"\]" ./conf/lighttpd.conf > /dev/null
			if [ $? -ne 0 ]; then
				echo "\$HTTP[\"remoteip\"] !~ \"${IP}\" {" >> ./conf/lighttpd.conf
				echo "url.access-deny = (\"\")" >> ./conf/lighttpd.conf
				echo "}" >> ./conf/lighttpd.conf
			else
				Replace "s/^\$HTTP\[\"remoteip\"\].*$/\$HTTP\[\"remoteip\"\] !~ \"$IP\" {/g" ./conf/lighttpd.conf
			fi
			print_log "|lighttpdstart|    |--allowed IP changed [$IP]"
		fi

		if [ -f /var/run/v3dld.dog.pid ]; then
			print_log "|lighttpdstart| v3dld watchdog ALREADY RUNNING or PID FILE EXIST"
			SIGKILL v3dld.dog "lighttpdstart"
		fi

		if [ -f "$LIGHTTPD_PID" ]; then
			print_log "|lighttpdstart| lighttpd ALREADY RUNNING or PID FILE EXIST"
			SIGKILL lighttpd "lighttpdstart"
		fi
		./bin/lighttpd -f ./conf/lighttpd.conf -m ./lib

		print_log "[lighttpdstart] starting v3dld watchdog"
		callWatchdog v3dld

		check_process "lighttpd" "lighttpdstart"

		print_log "[lighttpdstart] STARTING LIGHTTPD(TARGET PROCESS) DONE\n"
	;;

	"lighttpdstop")
		print_log "============================================================"
		print_log "|lighttpdstop| STOP LIGHTTPD(TARGET PROCESS)"
		#echo "terminating the lighttpd..."

		if [ -f /var/run/v3dld.dog.pid ]; then
			SIGKILL v3dld.dog "lighttpdstop"
		fi
		delete_file_check_file "/var/run/v3dld.dog" "lighttpdstop"

		print_log "|lighttpdstop| STOP LIGHTTPD"
		if [ -f "$LIGHTTPD_PID" ]; then
			SIGKILL lighttpd "lighttpdstop"
		fi
		delete_file_check_file "$LIGHTTPD_PID" "lighttpdstop"

		check_process "lighttpd"  "lighttpdstop"
		print_log "|lighttpdstop| STOP LIGHTTPD(TARGET PROCESS) DONE\n"
		print_watchdog_log "LIGHTTPD(TARGET PROCESS) STOPPED [FROM v3dl.sh]"
	;;
	"v3dldstart")
		print_log "============================================================"
		print_log "|v3dldstart| STARTING V3DLD(TARGET PROCESS)"
		print_watchdog_log "STARTING V3DLD(TARGET PROCESS) [FROM v3dl.sh]"
		./bin/migration.sh
		#echo "Starting the v3dld..."

		CheckDistributeID v3dldstart

		# v3dld
		if [ -f /var/run/v3dld.pid ]; then
			SIGTERM v3dld "v3dldstart"
		fi
		./bin/v3dld -f ./conf/v3dld.conf

		# v3dl-agentd
		print_log "|v3dldstart| STARTING V3DL-AGENTD"
		if [ -f /var/run/v3dl-agentd.pid ]; then
			SIGKILL v3dl-agentd "v3dldstart"
		fi
		./bin/v3dl-agentd -f ./conf/v3dld.conf

		# v3filemond
		if [ "`uname`" = "Linux" ]; then
			if [ "`CheckNotifySupport`" = "fanotifybase" ]; then
				V3FILEMOND=v3fbmond
			fi

			if [ "_$V3FILEMOND" != "_" ]; then
				print_log "|v3dldstart| STARTING V3FBMOND"
				if [ -f /var/run/$V3FILEMOND.pid ]; then
					SIGTERM $V3FILEMOND "v3dldstart"
				fi
				./bin/$V3FILEMOND &
			fi
		fi

		# APC_AGENT_BINARY
		if [ "$APC_USE" != "No" ]; then
			if [ -f /var/run/apc-agentd.pid ]; then
				print_log "|v3dldstart| [apc-agentd] ALREADY RUNNING or PID FILE EXIST"
            	SIGKILL apc-agentd "v3dldstart"
			fi
			if [ -f /var/run/epp-agentd.pid ]; then
				print_log "|v3dldstart| [epp-agentd] ALREADY RUNNING or PID FILE EXIST"
				SIGTERM epp-agentd "v3dldstart"
			fi
			if [ -f /var/run/gpms-agentd.pid ]; then
				if [ "`func_check_gooroom`" = "true" ]; then
					print_log "|v3dldstart| [gpms-agentd] ALREADY RUNNING or PID FILE EXIST"
					SIGTERM gpms-agentd "v3dldstart"
				fi
			fi
			print_log "|v3dldstart| STARTING $APC_AGENT_BINARY"
			if [ "$APC_AGENT_BINARY" = "gpms-agentd" ]; then
				if [ "`func_check_gooroom`" = "true" ]; then
					./bin/$APC_AGENT_BINARY &
				fi
			else
				./bin/$APC_AGENT_BINARY -f ./conf/v3dld.conf
			fi

			if [ -f /var/run/apc-agentd.dog.pid ]; then
				print_log "|v3dldstart| [apc-agentd] watchdog ALREADY RUNNING or PID FILE EXIST"
                SIGKILL apc-agentd.dog "v3dldstart"
			fi
			if [ -f /var/run/epp-agentd.dog.pid ]; then
				print_log "|v3dldstart| [epp-agentd] watchdog ALREADY RUNNING or PID FILE EXIST"
            	SIGKILL epp-agentd.dog "v3dldstart"
			fi
			if [ -f /var/run/gpms-agentd.dog.pid ]; then
				if [ "`func_check_gooroom`" = "true" ]; then
					print_log "|v3dldstart| [gpms-agentd] watchdog ALREADY RUNNING or PID FILE EXIST"
					SIGKILL gpms-agentd.dog "v3dldstart"
				fi
			fi

			# watchdog - APC_AGENT_BINARY
			print_log "|v3dldstart| STARTING ${APC_AGENTD_BINARY} watchdog"
			callWatchdog $APC_AGENT_BINARY
		fi

		# watchdog - v3dld
		print_log "|v3dldstart| STARTING v3dld watchdog"
		if [ -f /var/run/v3dld.dog.pid ]; then
			SIGKILL v3dld.dog "v3dldstart"
		fi
		callWatchdog v3dld

		# watchdog - v3dl-agentd
		print_log "|v3dldstart| STARTING v3dl-agentd watchdog"
		if [ -f /var/run/v3dl-agentd.dog.pid ]; then
			SIGKILL v3dl-agentd.dog "v3dldstart"
		fi
		callWatchdog v3dl-agentd

		# watchdog - v3filemond
		if [ "`uname`" = "Linux" ]; then
			if [ "`CheckNotifySupport`" = "fanotifybase" ]; then
				V3FILEMOND=v3fbmond
			fi

			if [ "_$V3FILEMOND" != "_" ]; then
				print_log "v3dldstart| STARTING V3FBMOND WATCHDOG"
				if [ -f /var/run/$V3FILEMOND.dog.pid ]; then
                    SIGKILL v3fbmond.dog "v3dldstart"
				fi
				callWatchdog $V3FILEMOND
			fi
		fi

		check_process v3dld  "v3dldstart"

		if [ "$APC_USE" != "No" ]; then
			check_process ${APC_AGENT_BINARY} "v3dldstart"
		fi

		check_process v3dl-agentd  "v3dldstart"

		if [ "_$V3FILEMOND" != "_" ]; then
			check_process ${V3FILEMOND}  "v3dldstart"
		fi

		print_log "|v3dldstart| STARTING V3DLD(TARGET PROCESS) DONE\n"

	;;
	"v3dldstop")
		print_log "============================================================"
		print_log "[v3dldstop] STOP V3DLD(TARGET PROCESS)"
		#echo "terminating the v3dld..."

		# watchdog - v3dld
		if [ -f /var/run/v3dld.dog.pid ]; then
			SIGKILL v3dld.dog "v3dldstop"
		fi
		delete_file_check_file "/var/run/v3dld.dog.pid" "v3dldstop"

		# watchdog - v3dl-agentd
		if [ -f /var/run/v3dl-agentd.dog.pid ]; then
			SIGKILL v3dl-agentd.dog "v3dldstop"
		fi
		delete_file_check_file "/var/run/v3dl-agentd.dog.pid" "v3dldstop"

		# watchdog - v3filemond
		if [ "`uname`" = "Linux" ]; then
			if [ "`CheckNotifySupport`" = "fanotifybase" ]; then
				V3FILEMOND=v3fbmond
			fi

			if [ "_$V3FILEMOND" != "_" ]; then
				if [ -f /var/run/$V3FILEMOND.dog.pid ]; then
                    SIGKILL v3fbmond.dog "v3dldstop"
				fi
            	delete_file_check_file "/var/run/$V3FILEMOND.dog.pid" "v3dldstop"
			fi
		fi

		# v3dl-agentd
		if [ -f /var/run/v3dl-agentd.pid ]; then
			SIGKILL v3dl-agentd "v3dldstop"
		fi
		delete_file_check_file "/var/run/v3dl-agentd.pid" "v3dldstop"

		if [ "$APC_USE" != "No" ]; then
			# watchdog - apc-agentd
			if [ -f /var/run/apc-agentd.dog.pid ]; then
            	SIGKILL apc-agentd.dog "v3dldstop"
			fi
			delete_file_check_file "/var/run/apc-agentd.dog.pid" "v3dldstop"

			# apc-agentd
			if [ -f /var/run/apc-agentd.pid ]; then
            	SIGKILL apc-agentd "v3dldstop"
			fi
			delete_file_check_file "/var/run/apc-agentd.pid" "v3dldstop"

			# watchdog - epp-agentd
			if [ -f /var/run/epp-agentd.dog.pid ]; then
            	SIGKILL epp-agentd.dog "v3dldstop"
			fi
			delete_file_check_file "/var/run/epp-agentd.dog.pid" "v3dldstop"

			# epp-agentd
			if [ -f /var/run/epp-agentd.pid ]; then
				SIGTERM epp-agentd "v3dldstop"
			fi
			delete_file_check_file "/var/run/epp-agentd.pid" "v3dldstop"

			#watchdog - gpms-agentd
			if [ -f /var/run/gpms-agentd.dog.pid ]; then
				if [ "`func_check_gooroom`" = "true" ]; then
					SIGKILL gpms-agentd.dog "v3dldstop"
				fi
				delete_file_check_file "/var/run/gpms-agentd.dog.pid" "v3dldstop"
			fi

			# gpms-agentd
			if [ -f /var/run/gpms-agentd.pid ]; then
				if [ "`func_check_gooroom`" = "true" ]; then
					SIGTERM gpms-agentd "v3dldstop"
				fi
				delete_file_check_file "/var/run/gpms-agentd.pid" "v3dldstop"
			fi
		fi

		# v3filemond
		if [ "`uname`" = "Linux" ]; then
			if [ "`CheckNotifySupport`" = "fanotifybase" ]; then
				V3FILEMOND=v3fbmond
			fi

			if [ "_$V3FILEMOND" != "_" ]; then
				if [ -f /var/run/$V3FILEMOND.pid ]; then
					SIGTERM $V3FILEMOND "v3dldstop"
				fi
            	delete_file_check_file "/var/run/$V3FILEMOND.pid" "v3dldstop"
			fi
		fi

		# v3dld
		if [ -f /var/run/v3dld.pid ]; then
			SIGTERM v3dld "v3dldstop"
		fi
		delete_file_check_file "/var/run/v3dld.pid" "v3dldstop"

		check_process v3dld  "v3dldstop"
		if [ "$APC_USE" != "No" ]; then
			check_process ${APC_AGENT_BINARY} "v3dldstop"
		fi

		check_process v3dl-agentd  "v3dldstop"
		if [ "_$V3FILEMOND" != "_" ]; then
			check_process ${V3FILEMOND}  "v3dldstop"
		fi

		print_log "[v3dldstop] STOP V3DLD(TARGET PROCESS) DONE\n"
		print_watchdog_log "V3DLD(TARGET PROCESS) STOPPED [FROM v3dl.sh]"
	;;
"watchdogstop")
		print_log "============================================================"
		print_log "[watchdogstop] STOP WATCHDOG PROCESSES"
		#echo "terminating the watchdog..."

		# watchdog - v3dld
		if [ -f /var/run/v3dld.dog.pid ]; then
			SIGKILL v3dld.dog "watchdogstop"
		fi
		delete_file_check_file "/var/run/v3dld.dog.pid" "watchdogstop"

		# watchdog - v3dl-agentd
		if [ -f /var/run/v3dl-agentd.dog.pid ]; then
			SIGKILL v3dl-agentd.dog "watchdogstop"
		fi
		delete_file_check_file "/var/run/v3dl-agentd.dog.pid" "watchdogstop"

		# watchdog - v3filemond
		if [ "`uname`" = "Linux" ]; then
			if [ "`CheckNotifySupport`" = "fanotifybase" ]; then
				V3FILEMOND=v3fbmond
			fi

			if [ "_$V3FILEMOND" != "_" ]; then
				if [ -f /var/run/$V3FILEMOND.dog.pid ]; then
                    SIGKILL v3fbmond.dog "watchdogstop"
				fi
                delete_file_check_file "/var/run/$V3FILEMOND.dog.pid" "watchdogstop"
			fi
		fi

		if [ "$APC_USE" != "No" ]; then
			# watchdog - apc-agentd / epp-agentd
			if [ -f /var/run/${APC_AGENT_BINARY}.dog.pid ]; then
            	SIGKILL ${APC_AGENT_BINARY}.dog "watchdogstop"
			fi
			delete_file_check_file "/var/run/${APC_AGENT_BINARY}.dog.pid" "watchdogstop"
		fi

		print_log "[watchdogstop] STOP WATCHDOG PROCESSES DONE\n"
		print_watchdog_log "WATCHDOG STOPPED [FROM v3dl.sh]"
	;;
	*)
		echo_error "Usage: ./v3dl.sh {start|stop|status}"
esac
