<?php

/**
 * @brief       php로그를 남긴다
 * @param[in]   level : 로그레벨
 * @param[in]   depth : BackTrace깊이설정(0인 경우 본 함수를 호출한 지점 기준으로 출력한다)
 * @param[in]   format : format 문자열
 * @return      없음.
 * @remark      없음.
 */
function PHPLOG($level, $depth, $format) {
    if(!file_exists(g_v3net_conf_file)) return;

    $conf = file_get_contents(g_v3net_conf_file);
    preg_match('/LogLevel=(?P<level>\w+)\s/i', $conf, $matches);
    if($matches["level"] < $level) return;

    $args = array_slice(func_get_args(), 3);
    $logLevelString = array("OFF", "ALERT", "FATAL", "ERROR", "WARNING", "NORMAL", "NOTICE", "INFORM", "DEBUG");

    $message  = "[".dateNow()."]";
    $message .= "[".$logLevelString[$level]."] ";

    /* 디버그 레벨인 경우 로그를 출력지점의 파일명과 라인번호를 함께 출력한다 */
    if ($level >= array_search("DEBUG", $logLevelString)) {
        $bt = debug_backtrace();
        $message .= $bt[$depth]["file"]."(".$bt[$depth]["line"].") : ";
    }

    $message .= vsprintf($format, array_values($args)) . "\n";

    $isputfile = true;
    $logPath = $GLOBALS["APP_DIR"]."../log";
    if(!file_exists($logPath)){
        if(!(mkdir($logPath) && chmod($logPath, 0750)))
            $isputfile = false;
    }

    if($isputfile)
        file_put_contents($logPath."/php_".date("Ymd").".log", $message, FILE_APPEND);
}

// 변수의 기본 null 치환
function nvl(&$value, $default = ""){
    if (!isset($value)) $value = $default;
    else if ($value == null || trim($value) == "") $value = $default;
    return trim($value);
}
function strhtml(&$value, $isBlank=true){
    if ($isBlank){
        if (!isset($value) || $value == null) return "&nbsp;";
        if (trim($value) == "") return "&nbsp;";
    }
    $value = $newstr = str_replace(array("\r\n", "\n", "\r"), " ", $value);
    return htmlspecialchars($value);
}
function strDb(&$value){
    if (!isset($value) || $value == null) return "";
    return trim(str_replace("'", "''", $value));
}

// 문자열 자르기
function cutstr($str, $len, $tail='…', $checkmb=true) {
  /*
  preg_match_all('/[\xEA-\xED][\x80-\xFF]{2}|./', $str, $match);

  $m    = $match[0];
  $slen = strlen($str);
  $tlen = strlen($tail);
  $mlen = count($m);

  if ($slen <= $len) return $str;
  if (!$checkmb && $mlen <= $len) return $str;

  $ret  = array();
  $count = 0;

  for ($i=0; $i < $mlen; $i++) {
    $count += ($checkmb && strlen($m[$i]) > 1)?2:1;
    if ($count + $tlen > $len) break;
    $ret[] = $m[$i];
  }
  return join('', $ret).$tail;
  */
  if (strlen($str) > $len) $str = mb_strimwidth($str, 0, $len, $tail, "utf-8");
  return $str ;
}
// page 계산 함수
function page_param(&$page, $total=1, $plist=20){
    $page = ($page < 1)?1:intval($page);
    $param[0] = ($total < 1)?1: ceil($total / $plist);
    if ($page >= $param[0]) $page = $param[0];
    $param[1] = ($page <= 1)?1:$page -1;
    $param[2] = ($page >= $param[0])?$param[0]:$page + 1;
    return $param;
}
// include 함수
function inc_func($incType, $incFile)
{
  switch(strtoupper($incType)){
    case g_inc_type_inc:
      include $GLOBALS["INC_DIR"] . $incFile . ".inc.php" ;
      break;
    case g_inc_type_var:
      global $g_msg_text;
      require $GLOBALS["VAR_DIR"] . $GLOBALS["g_msg_type"] . "/" . $incFile . ".var.php";
      break;
    case g_inc_type_db:
      include $GLOBALS["DB_DIR"] . $incFile . ".db.php";
      break;
    default:
      break;
  }
}

function isOsLinux()
{
    return (strpos(strtolower(PHP_OS), 'linux') !== false);
}

function strOptionValue($startIdx, $lastIdx, $selectTime=0, $strPad="")
{
    $rtnStr = "";
    $isSelect = " selected";
    $strText = "";
    do{
        $strText = ($strPad == "")?$startIdx:str_pad($startIdx, 2, $strPad, STR_PAD_LEFT);
        $isSelect = ($selectTime == $startIdx)? " selected" : "";
        $rtnStr .= "<option ".$isSelect." value=\"".$startIdx."\">".$strText."</option>";
        $startIdx++;
    }while($startIdx < $lastIdx);
    return $rtnStr;
}

function replaceSpc(&$str){
    $fromChar = array("\\", "\"", "[", "]", "{", "}");
    $toChar = array("\\\\", "\\\"", "［", "］", "｛", "｝");
    return str_replace($fromChar, $toChar, $str);
}
function dateNow(){
    return date("Y-m-d H:i:s");
}
function makeAxisX($axisData, $axisCnt=6){
    $baseValue = 1;
    $avgValue = 0;
    $maxValue = 0;
    $avgUp = 0;
    $avgDown = 0;
    $axisX = array();
    $axisY = array();
    // foreach($axisData as $key=>$value) $axisY[$key] = 0;
    $maxValue = max($axisData);
    if ($maxValue > ($baseValue * ($axisCnt -1))) {
        $baseValue = ceil($maxValue / ($axisCnt -1));
        $expValue = strlen($baseValue) -1;
        if ($expValue > 0){
            $powValue = pow(10, $expValue);
            for($i = 1; $i <= 10; $i++){
                if ($baseValue < ($i * $powValue)){
                    $baseValue = ($i * $powValue);
                    break;
                }
            }
        }
    }
    for($i = 0, $j = ($axisCnt-1); $i < $axisCnt; $i++, $j--) $axisX[$j] = $baseValue * $i;
    foreach($axisData as $key => $value) {
        $axisY[$key] = ceil(($value / $axisX[0]) * 100);
        if ($axisY[$key] < 1) $axisY[$key] = 0;
        $axisY2[$key] = 100 - $axisY[$key];
    }
    ksort($axisX);
    return array($axisX, $axisY, $axisY2);
}

function chkDate($orgDate){
    $tmp = split("-", $orgDate, 3);
    $nYear = intval($tmp[0]);
    $nMonth = intval($tmp[1]);
    $nDay = intval($tmp[2]);
    return checkDate($nMonth, $nDay, $nYear);
}

function replaceArgsToString($text, $args=array()) {
    if(sizeof($args) > 0) {
        foreach($args as $key => $value) {
            $findKey = "%".($key+1)."p";
            $text = str_replace($findKey, $value, $text);
        }
    }

    return $text;
}

function makeEventLog($eventId, $logId, $args=array()){
    global $g_msg_text, $g_fss;
    inc_func(g_inc_type_var, "msg_" . $eventId);

    $logLevle = $GLOBALS["g_msg_text"][$eventId][$logId][0];
    $logType = "";
    switch($eventId){
    case "event_log_4000":
        switch($logId){
        case "4001":
            $logType = "12901";
            break;
        case "4011":
        case "4002":
            $logType = "12902";
            break;
        case "4003":
            $logType = "12903";
            break;
        }
        break;
    case "event_log_2000":
        switch($logId){
        case "2001":
            $logType = "10501";
            break;
        case "2002":
            $logType = "10503";
            break;
        case "2011":
		case "2020":
		case "2412":
            $logType = "10502";
            break;
        }
        break;
        case "event_log_3000":
            switch($logId){
            case "3009":
                $logType = "10501";
                break;
            case "3010":
                $logType = "10502";
                break;
            }
    }

    $logArgs = '';
    if(count($args) > 0) {
        for($i = 0; $i <= count($args) - 1; $i++) {
            $logArgs .= $args[$i].'||';
        }
        $logArgs .= $args[$i];
    }

    PHPLOG(g_LOG_DEBUG, 0, "makeEventLog: [".$logLevle.",".$logType.",".$logId.",".$logArgs."]");

    $cmdStr = "EVENT_LOG";
    $inData = pack("ia8a8a*", $logLevle, $logType, $logId, $logArgs);
    if ($g_fss->sendCmd($cmdStr,$inData))
    {
        $resData = $g_fss->readCmd($cmdStr);
    }
}

function loginSessionClear(){
    unset($_SESSION["LOGIN_ID"]);
    unset($_SESSION["LOGIN_SESSION"]);
    session_destroy();

    if (ini_get("session.use_cookies")) {
        setcookie(session_name(), '', time() - 42000);
    }
}

function getWebSecurity() {
    if(!file_exists(g_v3net_conf_file)) return;
    $cont = file_get_contents(g_v3net_conf_file);
    preg_match('/WebSecurity=(?P<webSecurity>\w+)/i', $cont, $matches);

    $rtn = g_web_security_off;
    $lowerWebSecurity = strtolower($matches["webSecurity"]);
    if($lowerWebSecurity == "yes") {
        $rtn = g_web_security_on;
    } else if($lowerWebSecurity == "no") {
        $rtn = g_web_security_off;
    } else {
        PHPLOG(g_LOG_WARNING, 0, "getWebSecurity() : Web Security(".$lowerWebSecurity.")");
    }

    return $rtn;
}

function setWebSecurity($on) {
    $webSecurityOpt = ($on == g_web_security_on) ? "Yes" : "No";
    $cont = file_get_contents(g_v3net_conf_file);

    $pos = strpos($cont, "WebSecurity");
    if($pos === false) {
        $newCont = $cont."\nWebSecurity=".$webSecurityOpt;
    } else {
        $newCont = preg_replace('/WebSecurity=\w+/i', 'WebSecurity='.$webSecurityOpt, $cont);
    }
    file_put_contents(g_v3net_conf_file, $newCont);
}

function checkSupportVersion($yourTarget, $minimumSupport) {
    $isSupport = false;

    $verYourTarget  = explode('.', $yourTarget);
    $verMinimumSupport = explode('.', $minimumSupport);

    $max = (count($verYourTarget) > count($verMinimumSupport))? count($verYourTarget) : count($verMinimumSupport);
    for($i = 0; $i < $max; $i++) {
        if(intval(nvl($verYourTarget[$i])) > intval(nvl($verMinimumSupport[$i]))) {
            $isSupport = true;
            break;
        } else if (intval(nvl($verYourTarget[$i])) < intval(nvl($verMinimumSupport[$i]))) {
            $isSupport = false;
            break;
        } else {
            $isSupport = true;
        }
    }

    return $isSupport;
}

function getServerPort() {
    if(!file_exists(g_lighttpd_conf_file)) return;
    $cont = file_get_contents(g_lighttpd_conf_file);
    $contrim = preg_replace('/\s+/', '', $cont);
    preg_match('/server.port=(?P<port>\w+)/i', $contrim, $matches);

    $port = (int)$matches["port"];
    PHPLOG(g_LOG_WARNING, 0, "getServerPort() : server_port [".$port."] ");

    if($port <= 0)
    {
        return ;
    }
    else
    {
        return $port;
    }

}

function getEULAAgreement() {
    if(!file_exists(g_v3net_conf_file)) return;
    $cont = file_get_contents(g_v3net_conf_file);
    preg_match('/EULA=(?P<eula>\w+)/i', $cont, $matches);

    $rtn = 1;
    $lowerEula = strtolower($matches["eula"]);
    if($lowerEula == "yes") {
        $rtn = 1;
    } else if($lowerEula == "no") {
        $rtn = 0;
    } else {
        PHPLOG(g_LOG_WARNING, 0, "getEulaAgreement() : Eula Agreement(".$lowerEula.")");
    }

    return $rtn;
}

function getProduct() {
    if(!file_exists(g_v3net_conf_file)) return;
    $cont = file_get_contents(g_v3net_conf_file);
    preg_match('/ProductName=(?P<product>\w+)/i', $cont, $matches);

    $rtn = 0;

    $PRODUCT = $matches["product"];
    if($PRODUCT == "V3_DESKTOP_FOR_LINUX")
    {
        $rtn = "v3dl";
    }
    else if($PRODUCT == "V3LITE_OPENOS")
    {
        $rtn = "v3ll";
    }
    else
    {
        PHPLOG(g_LOG_WARNING, 0, "getProduct() : Product is [".$PRODUCT."]");
    }

    return $rtn;

}
?>
